package footprint;

use lib "./Modules";
use strict;
use exons;
use interval;

#  takes: not Areg, but just ONE alignment intervall, the alignment
#         and the minimal length of a motif to return        
#returns: a list  of hashes, which hold the sequence(key)
#         and first and last alignment position of aligned fragments (value)
#!! the inforamtion cannot be written in a Areg-like data structure
#!! lower case letter are treaten like gaps
sub select_concom {
  my ($regref, $aliref, $AtoSref, $StoAref, $minlen)= @_;
  my @list;
  foreach my $seq (keys(%$aliref)) {
    my $b;
    my $e;
#    #find first UC letter in sequence fragment
#    #this is only needed, if lc letters are not indexed als gaps
#    for (my $i=$regref->[0]-1; $i<=$regref->[1]-1; $i++) {
#      if ($aliref->{$seq}->[$i] =~ m/[A-Z]/) {
#	$b=$i+1;
#	last;
#      }
#    }
#    #find last UC letter in sequence fragment
#    for (my $i=$regref->[1]-1; $i>=$regref->[0]-1; $i--) {
#      if ($aliref->{$seq}->[$i] =~ m/[A-Z]/) {
#	$e=$i+1;
#	last;
#      }
#    }
    if ($AtoSref->{$seq}->[$regref->[0]] =~ m/\d*,(\d*)/) {
      $b = $StoAref->{$seq}->[$1];
    }
    else {$b = $regref->[0]} 
    if ($AtoSref->{$seq}->[$regref->[1]] =~ m/(\d*),\d*/) {
      $e = $StoAref->{$seq}->[$1];
    }
    else {$e = $regref->[1]} 

#    print "Areg($seq):$regref->[0]-$regref->[1]||$b-$e|\n";

    #ensure that everything is OK with the positions $b and $e
    if (defined($b) & defined($e)) {
      if ($b == '0') {next}
      if ($e == '0') {next}
      if ($e-$b+1 >= $minlen) {
	push @list, [$b,$e,$seq];
      }
    }
    else {
      if (defined($b) | defined($e)) {
#	die "Exit BigMurx: Something very, very bad happened!\n";
      }
      else {
	print "No upper case letters.\n";
      }
    }
  }
  #take the list of fragments and see if the intervalls are still overlapping
  #(1) write @list into an other data stricture, @ds:
  #    [[Apos, 0/1, ref_to_element_of_@list],[...],...]
  #    while Apos is either $b or $e of an intervall
  #          0/1 is 0 for $b (auf) and 1 for $e (zu) 
  my @ds;
  foreach my $i (@list) {
    push @ds, [$i->[0], 0, \$i];
    push @ds, [$i->[1], 1, \$i];
  }
  #(2) sort @ds ascending by pos
  my @dsort = sort { $a->[0] <=> $b->[0] } @ds;
  
  #(3) find connected components (overlapping intervalls), @concom
  my $aufzu=0;
  my $lastpos=0;
  my @concom;

  foreach my $pos (@dsort) {
    #error
#    print "pos=[$pos->[0]][$pos->[1]][$pos->[2]]\n";
    if ($aufzu < 0) {
      die "Exit Megamurx: internal failure, program surgery required!\n";
    }
    #close the connected component and open an new one
    #if $aufzu == 0 is not just an intermediate state
    elsif ($aufzu == 0) {
      unless ($pos->[0] == $lastpos) {
	push @concom, []; #build a new concom
      }
    }
    #open and close intervalls, and add intervalls to the connected component
    if ($aufzu >= 0) {
      $lastpos=$pos->[0];
      if ($pos->[1] == 0) {
	$aufzu++;
	push @{$concom[$#concom]}, $pos->[2];
      }
      else { $aufzu--; }
    }
  }
  unless ($aufzu == 0) {
    die "Exit Murx: internal failure, not necessarely a big problem!\n";
  }

  #pack connected components in data structure (analogous to Agap)
  #com: this is NOT adjustetd for multiple fragments
  #     per sequence and connected component!
  my @back;
  foreach my $i (@concom) {
    my %tmp;
    foreach my $j (@$i) {
      if (exists($tmp{$$j->[2]})) {
	die "Exit Unfinished: construction site!\n";
      }
      else {
	$tmp{$$j->[2]}=[$$j->[0],$$j->[1]];
      }
    }
    push @back, \%tmp;
  }

  #for debuging
  #foreach my $i (@back) {
  #  print "[>concom:\n";
  #  print_Agap($i);
  #}

  return \@back;
}

sub print_Agap {
  my $Agapref=shift;
  foreach my $seq (sort(keys(%$Agapref))) {
    printf "> %-10s\n", $seq;
    print "@{$Agapref->{$seq}}";
    print "\n";
  }
  print "\n\n";
  return;
}

sub fragmentali {
  my ($smotropyref, $cut, $alilen) = @_;
  my @alifrag;
  my $inblock=0;
  for (my $Apos=1; $Apos<scalar(@$smotropyref); $Apos++) {
    my $bigger = 0;
    if ($smotropyref->[$Apos] >= $cut) {
      $bigger = 1;
    }
    if ($inblock == 0) {
      if ($bigger) {
	push @alifrag, [$Apos];
	$inblock = 1;
      }
    }
    else {
      if ($bigger == 0) {
	my $len = $Apos-$alifrag[$#alifrag]->[0];
	push @{$alifrag[$#alifrag]}, ($Apos-1, $len);
	$inblock = 0;
      }
    }
  }
  if (scalar(@{$alifrag[$#alifrag]}) == 1) {
    my $len = $alilen-$alifrag[$#alifrag]->[0]+1;
    push @{$alifrag[$#alifrag]}, ($alilen, $len);
  }

  return \@alifrag;
}

sub select_alifrag {
  my ($alifragref, $minlen) = @_;
  my @select;
  foreach my $frag (@$alifragref) {
    if ($frag->[2] >= $minlen) {
      push @select, $frag;
    }
  }
  return \@select;
}

sub select_alifrag2 {
  my ($alifragref, $StoAref, $exonsref) = @_;
  my @select;
  my $Aregref = exons::exons2Areg($StoAref, $exonsref);
  foreach my $frag (@$alifragref) {
    my $doselect=1;
    #print " frag:@$frag\n";
    foreach my $reg (@$Aregref) {
      #print "exons:@$reg\n";
      if (interval::overlap($frag->[0], $frag->[1], @$reg)) {
	$doselect=0;
	last;
      }
    }
    if ($doselect) {
      push @select, $frag;
    }
  }
  return \@select;
}

sub print_Areg2multifasta2 {
  my ($aliref, $Aregref, $AtoSref, $StoAref, $namesref, $minlen, $string) = @_;
  my $outfile='motifs.mfa';
  my %StoM;
  my %MtoS;
  my $M=0;  #Motife number
  open OUT, ">$outfile" or die "Can\'t write file \'$outfile\'!\n";
  print STDERR "Writing motifs to \'$outfile\'...\n";
  foreach my $reg (@$Aregref) {
    my $testref = select_concom($reg, $aliref, $AtoSref, $StoAref, $minlen);
    foreach my $cc (@$testref) {
      $M++;
      foreach my $seq (keys(%$AtoSref)) {
	my $name;
	if ($namesref->[1]->{$seq} eq "") {
	  $name = $seq;
	}
	else {
	  $name = $namesref->[1]->{$seq};
	}
	unless (exists($cc->{$seq})) {
	  next;
	}
	my $Avon=$cc->{$seq}->[0];
	my $Abis=$cc->{$seq}->[1];
	my $Svon=$AtoSref->{$seq}->[$Avon];
	my $Sbis=$AtoSref->{$seq}->[$Abis];
	my $fastaheader = "M" . $M . "-" . $name . " A:" . $Avon . "-" .
	    $Abis . " S:" . $Svon . "-" . $Sbis . " $string"; 
	print OUT "> $fastaheader\n";
	foreach my $l (@{$aliref->{$seq}}[$Avon-1..$Abis-1]) {
	  print OUT "$l" unless $l eq $main::gap;
	}
	print OUT "\n\n";
      }
    }
  }
  close OUT;
}


sub print_Areg2multifasta {
  my ($aliref, $Aregref, $AtoSref, $namesref, $minlen) = @_;
  my $outfile='motifs.mfa';
  my %StoM;
  my %MtoS;
  my $M=0;  #Motife number
  open OUT, ">$outfile" or die "Can\'t write file \'$outfile\'!\n";
  print STDERR "Writing motifs to \'$outfile\'...\n";
  foreach my $reg (@$Aregref) {
    $M++;
    foreach my $seq (keys(%$AtoSref)) {
      my $name;
      if ($namesref->[1]->{$seq} eq "") {
	$name = $seq;
      }
      else {
	$name = $namesref->[1]->{$seq};
      }
      my $Svon=$AtoSref->{$seq}->[$reg->[0]];
      my $Sbis=$AtoSref->{$seq}->[$reg->[1]];
      if ($Sbis-$Svon+1 < $minlen) {next}
      my $fastaheader = "M" . $M . "-" . $name . " A:" . $reg->[0] . "-" .
	  $reg->[1] . " S:" . $Svon . "-" . $Sbis; 
      print OUT "> $fastaheader\n";
      foreach my $l (@{$aliref->{$seq}}[$reg->[0]-1..$reg->[1]-1]) {
	print OUT "$l" unless $l eq $main::gap;
      }
      print OUT "\n\n";
    }
  }
}

#do write this method! this is just a copy of print_Areg2multifasta
sub print_Areg2bbqin {
#  return;
}

sub print_concali2
{
  my ($aliref, $AtoSref, $StoAref, $Aregref, $minlen, $string) = @_;
  my %concali;
  foreach my $reg (@$Aregref) {
    my $testref = select_concom($reg, $aliref, $AtoSref, $StoAref, $minlen);
    foreach my $cc (@$testref) {
      my $Avon;
      my $Abis;
      foreach my $seq (keys(%$aliref)) {
	unless (exists($cc->{$seq})) {next}
	#print "[$seq]>" . $cc->{$seq}->[0] . "-" . $cc->{$seq}->[1] . "<\n";
	if (defined($Avon) == 0 | 
	    $Avon > $cc->{$seq}->[0]) {
	  $Avon=$cc->{$seq}->[0];
	}
	if (defined($Abis) == 0 |
	    $Abis < $cc->{$seq}->[1]) {
	  $Abis=$cc->{$seq}->[1];
	}
      }
      #print STDERR "Avon-Abis=>$Avon-$Abis<\n";
      foreach my $seq (keys(%$aliref)) {
	foreach my $l (@{$aliref->{$seq}}[$Avon-1..$Abis-1]) {
	  if ($l =~ m/[A-Z]/) {
	    push @{$concali{$seq}}, $l;
	  }
	  else {
	    push @{$concali{$seq}}, $main::gap;
	  }
	}
      }
    }
  }
  my $outfile = 'concali.fa';
  open (OUT, ">$outfile") or die "Can\'t write file \'$outfile\'!\n";
  print STDERR "Writing concatenated fragments to \'$outfile\'...\n";
  foreach my $seq (keys(%concali)) {
      print OUT "> $seq concatenated fragments, ";
      print OUT "mAAna_tropy $string\n";
      print OUT join('', @{$concali{$seq}}) . "\n\n";
  }
  close OUT;
  return;
}

#used by mAAna_noisy
sub print_concali
{
  my ($aliref, $Aregref, $string) = @_;
  my %concali;
  foreach my $reg (@$Aregref) {
    foreach my $seq (keys(%$aliref)) {
      push @{$concali{$seq}}, @{$aliref->{$seq}}[$reg->[0]-1..$reg->[1]-1];
    }
  }
  my $outfile = 'concali.fa';
  open (OUT, ">$outfile") or die "Can\'t write file \'$outfile\'!\n";
  print STDERR "Writing concatenated fragments to \'$outfile\'...\n";
  foreach my $seq (keys(%concali)) {
      print OUT "> $seq concatenated fragments, ";
      print OUT "mAAna_tropy $string\n";
      print OUT join('', @{$concali{$seq}}) . "\n\n";
  }
  close OUT;
  return;
}

#modified from mAAna.pl! 
sub print_tex2
{
  my ($aliref, $Aregref, $namesref, $AtoSref, $StoAref, $minlen, $string) = @_;
  my $count=0;
#  print "% $string\n";
  print "Footprint ";
  foreach my $seq (sort(keys(%{$namesref->[1]}))) {
    print "&\n\\multicolumn{2}{c|}{ " , $namesref->[1]->{$seq}  , " } ";
  }
  print "\\\\\n\\hline\n";
  
  foreach my $reg (@$Aregref) {
    #test, test, test

    my $testref = select_concom($reg, $aliref, $AtoSref, $StoAref, $minlen);
    foreach my $cc (@$testref) {
      $count++;
      printf "%3d ", $count;
      foreach my $nseq (sort(keys(%{$namesref->[1]}))) {
	my $seq = $namesref->[1]->{$nseq};
	if (exists($cc->{$seq})) {
	  my $pos = $AtoSref->{$seq}->[$cc->{$seq}->[0]];
	  my $len = $AtoSref->{$seq}->[$cc->{$seq}->[1]] - $pos + 1;
	  printf "&%7d &%5d ", $pos, $len; 
	}
	else {
	  print "&        &      ";
	}
      }
      print "\\\\\n";
    }
  }
  print "\\hline\n";
}


sub print_html
{
  my ($aliref, $Aregref, $namesref, $AtoSref, $StoAref, $minlen, $string) = @_;
  my $count=0;

  print "<html>\n<head>\n<title>Footprints</title>\n</head>\n<body>\n";
  print "<p>$string<\p>\n";
  print "<table border=\"1\">\n";
  print "<tr>\n";
  print "<td>Footprint</td>\n";
  foreach my $seq (sort(keys(%{$namesref->[1]}))) {
    print "  <td colspan=\"2\">" , $namesref->[1]->{$seq}  , "</td>\n";
  }
  print "</tr>\n";
  
  foreach my $reg (@$Aregref) {
    my $testref = select_concom($reg, $aliref, $AtoSref, $StoAref, $minlen);
    foreach my $cc (@$testref) {
      $count++;
      printf "<tr>\n  <td align=\"right\">%d</td>\n", $count;
      foreach my $nseq (sort(keys(%{$namesref->[1]}))) {
	my $seq = $namesref->[1]->{$nseq};
	if (exists($cc->{$seq})) {
	  my $pos = $AtoSref->{$seq}->[$cc->{$seq}->[0]];
	  my $len = $AtoSref->{$seq}->[$cc->{$seq}->[1]] - $pos + 1;
          printf "  <td>%d</td>\n  <td>%d</td>\n", $pos, $len;
	}
	else {
          print "  <td>&nbsp;</td>\n  <td>&nbsp;</td>\n";
	}
      }
      print "</tr>\n";
    }
  }
  print "</table>\n</body>\n</html>\n";
}


#modified from mAAna.pl! 
sub print_tex
{
  my ($aliref, $Aregref, $namesref, $AtoSref, $StoAref, $minlen) = @_;
  my $count=0;
  print "Footprint ";
  foreach my $seq (sort(keys(%{$namesref->[1]}))) {
    print "&\n\\multicolumn{2}{c|}{ " , $namesref->[1]->{$seq}  , " } ";
  }
  print "\\\\\n\\hline\n";

  foreach my $reg (@$Aregref) {
    #test, test, test
    print "region=>$reg->[0]<>$reg->[1]<\n";
    my $testref = select_concom($reg, $aliref, $AtoSref, $StoAref, $minlen);
    #end test
    $count++;
    my @list;
    foreach my $seq (keys(%$aliref)) {
      foreach my $char ($reg->[0]-1..$reg->[1]-1) {
	if ($aliref->{$seq}->[$char] =~ m/[A-Z]/) {
	  push @list, $seq;
	  #print "$seq is part of frag $count\n";
	  last;
	}
      }
    }
    my %list;
    foreach my $seq (@list) {
      $list{$seq}=[$AtoSref->{$seq}->[$reg->[0]], 
		   $AtoSref->{$seq}->[$reg->[1]]];
    }
    printf "%3d ", $count;
    foreach my $nseq (sort(keys(%{$namesref->[1]}))) {
      my $seq = $namesref->[1]->{$nseq};
      my $pos = $list{$seq}->[0];
      my $length = $list{$seq}->[1];
      if ($pos =~ m/\.5$/) {
	$pos += 0.5;
      }
      if ($length =~ m/\.5$/) {
	$length -= 0.5;
      }
      $length -= ($pos-1) ;
      if ($pos eq '') {
	print "&        &      ";
      }
      else {
	printf "&%7d &%5d ", $pos, $length;
      }
    }
    print "\\\\\n";
  }
  print "\\hline\n";
}

sub frag_L_dist {
  my $alifragref = shift;
  my %L_dist;
  foreach my $frag (@$alifragref) {
    $L_dist{$frag->[2]}++;
  }
  return \%L_dist;
}

sub print_L_dist {
  my $L_distref = shift;
  print "BEGIN--- L_dist:\n";
  my @sort = sort {$a <=> $b} keys(%$L_distref);
  foreach my $key (@sort) {
    printf "%4d : %7d\n" , $key, $L_distref->{$key};
  }
  print "END----- L_dist:\n";
}

sub print_alifrag {
  my $alifragref = shift;
  print "BEGIN--- alifrag:\n";
  foreach my $frag (@$alifragref) {
    print "$frag->[0] - $frag->[1]\n";
  }
  print "END----- alifrag:\n";
}

sub print_Areg {
    my $Aregref=shift;
    foreach my $ival (@$Aregref) {
	print "[ " . $ival->[0] . ", " . $ival->[1] . "]\n";
    }
}

#invertes Areg and alifrag
#returns a inverted version of Areg (without length at [2])
sub invert_Areg {
  my ($Aregref, $alilen) = @_;
  my @invAreg = [1];
  foreach my $ival (@$Aregref) {
    push @{$invAreg[$#invAreg]}, $ival->[0]-1;
    push @invAreg, [$ival->[1]+1];
  }
  push @{$invAreg[$#invAreg]}, $alilen;
  return \@invAreg;
}

#invertes Areg-like lists of lists with [0]=beg, [1]=end, [3]=name
#returns a inverted version of Areg, again with names at [3]
sub invert_Areg2 {
  my ($Aregref, $alilen) = @_;
  my $nj= '<->'; #nj = NameJoint
  my $beg='5prime';
  my $end='3prime';
  my $name=$beg;
  my @invAreg = [1];
  foreach my $ival (@$Aregref) {
    $name=$name.$nj.($ival->[2]);
    push @{$invAreg[$#invAreg]}, $ival->[0]-1, $name;
    push @invAreg, [$ival->[1]+1]; #set begin of next interval
    $name=$ival->[2];              #set begin of next name
  }
  if ($invAreg[$#invAreg]->[0] == '$alilen' | defined($alilen)==0 ) {
      pop @invAreg;
  }
  else {
      push @{$invAreg[$#invAreg]}, $alilen, $end;
  }
  return \@invAreg;
}


#  takes: a file location of a tracker like tex output for footprints 
#returns: one hash: key = sequence names, value = list of footprints
#         the list holds at [footprint] a two item list ($start, $len)
#         footprint
#         one array: hold the footprint name for each footprint in the
#                    list of footprints
sub read_tex {
  my $texfile = shift;
  my %footlist;
  my @footname;
  open (IN, "<$texfile") or die "Can\'t open >$texfile< for reading!\n";
  my @mom;
  my $count=0;
  while (<IN>) {
    if ($_ =~ m/^\\multicolumn\{2\}\{c\|\}\{\s(\S+)\s\}\s*\\\\/) {
      push @mom, $1; last;
    }
    elsif ($_ =~ m/^\\multicolumn\{2\}\{c\|\}\{\s(\S+)\s\}/) {
      push @mom, $1;
    }
  }
  if ($main::debug) {print "mom: @mom\n";}
  while (<IN>) {
    if ($_ =~ m/Clique/) { last }
    unless ( $_ =~ /&/) { next }
    my @line = split /&/;
    if ( scalar(@mom)*2+1 == scalar(@line) & $line[0] =~ m/^[^%]\s*(\S+)/) {
      $count++;
      $footname[$count]=$1;
      
      #modify last element of the line
      if ($line[$#line] =~ m/^\s*\\\\/) {
	$line[$#line] = '';
      }
      elsif ($line[$#line] =~ m/^\s*(\d+)/ ) {
	$line[$#line] = $1;
      }
      for(my $i=0; $i<scalar(@mom); $i++ ) {
	if ($line[2*$i+1] =~ m/^\s*$/) {
	  ${$footlist{$mom[$i]}}[$count]=['', ''];

	}
	elsif ($line[2*$i+1] =~ m/^\s*(\d+)\s*$/) {
	  my $start = $1;
	  my $len;
	  if ($line[2*$i+2] =~ m/^\s*(\d+)\s*$/) {
	    $len = $1;
	  }    
	  else {
	    die "Footprint without length? Footprint >$count<\n";
	  }
	  #print "key=$mom[$i] pos=$count start=$start len=$len\n";
	  ${$footlist{$mom[$i]}}[$count]=[$start,$len];
	}
	else {
	  print STDERR "Wos is do los!?\m";
	}
      }
    }
  }
  close IN;
  return \%footlist, \@footname;
}

#  takes: a file location of a tracker like tex output for footprints 
#returns: one hash: key = sequence names, value = list of footprints
#         the list holds a three item list ($start, $end, $name)
sub read_tex_2 {
  my $texfile = shift;
  my %footlist;
  open (IN, "<$texfile") or die "Can\'t open >$texfile< for reading!\n";
  my @mom;
  my $count=0;
  while (<IN>) {
    if ($_ =~ m/^\\multicolumn\{2\}\{c\|\}\{\s(\S+)\s\}\s*\\\\/) {
      push @mom, $1; last;
    }
    elsif ($_ =~ m/^\\multicolumn\{2\}\{c\|\}\{\s(\S+)\s\}/) {
      push @mom, $1;
    }
  }
  if ($main::debug) {print "mom: @mom\n";}
  while (<IN>) {
    if ($_ =~ m/Clique/) { last }
    unless ( $_ =~ /&/) { next }
    my @line = split /&/;
    if ( scalar(@mom)*2+1 == scalar(@line) & $line[0] =~ m/^\s*(\S+)/) {
      $count++;
      my $footname=$1;
      
      #modify last element of the line
      if ($line[$#line] =~ m/^\s*\\\\/) {
	$line[$#line] = '';
      }
      elsif ($line[$#line] =~ m/^\s*(\d+)/ ) {
	$line[$#line] = $1;
      }
      for(my $i=0; $i<scalar(@mom); $i++ ) {
	if ($line[2*$i+1] =~ m/^\s*$/) {
	  #${$footlist{$mom[$i]}}[$count]=['', ''];
	}
	elsif ($line[2*$i+1] =~ m/^\s*(\d+)\s*$/) {
	  my $start = $1;
	  my $len;
	  if ($line[2*$i+2] =~ m/^\s*(\d+)\s*$/) {
	    $len = $1;
	  }    
	  else {
	    die "Footprint without length? Footprint >$count<\n";
	  }
	  #print "key=$mom[$i] pos=$count start=$start len=$len\n";
	  push @{$footlist{$mom[$i]}}, [$start,$start+$len-1,$footname];
	}
	else {
	  print STDERR "Wos is do los!?\m";
	}
      }
    }
  }
  close IN;
  return \%footlist;
}

#  takes: Ref to hash 'footlist' (as returned by read_tex_2(!))
#         Ref to hash of regions like IGRs (as returned by IGRs(!)) 
#returns: Ref to hash: key=sequence name, value=3 item list
#          [0]=5prime gene, [1]=3prime gene, [2]=# footprints in between
#         Ref to hash: key=sequence name, value=3 item list
#          [0]=5prime gene, [1]=3prime gene, [2]=length footprints in between
sub foot2in_reg {
  my ($footlist, $regref)=@_;

  my %numfootreg;
  my %lenfootreg;
  my %thefootreg;
  
  foreach my $seq (keys(%$footlist)) {
    foreach my $reg (@{$regref->{$seq}}) {
      my $fprime; my $tprime;
      if ($reg->[2] =~ m/^(\S+)<->(\S+)$/) {
	$fprime=$1;
	$tprime=$2;
      }
      else {
	die "Sorry, I don't understand this name \'$reg->[2]\'\n";
      }
      push @{$numfootreg{$seq}}, [$fprime, $tprime];
      push @{$lenfootreg{$seq}}, [$fprime, $tprime];
      foreach my $foot (@{$footlist->{$seq}}) {
	if (interval::intersect($reg->[0],$reg->[1],$foot->[0],$foot->[1])) {
	  ${${$numfootreg{$seq}}[$#{$numfootreg{$seq}}]}[2]+=1;
	  my ($be)=interval::intersect($reg->[0],$reg->[1],$foot->[0],$foot->[1]);
	  ${${$lenfootreg{$seq}}[$#{$lenfootreg{$seq}}]}[2]+=$be->[1]-$be->[0]+1;
	  push @{$thefootreg{$seq}}, [$be->[0], $be->[1], $reg->[2]];
	}
      }
    }
  }
  return \%numfootreg, \%lenfootreg, \%thefootreg;
}

#  takes: ref to footlist (as results from read_tex)
#         the name of a sequence
#returns: -
sub print_col_from_tex {
  my ($footlistref, $seq) = @_;
  my $count=-1;
  my $sum;
  foreach my $footprint (@{$footlistref->{$seq}}) {
    $count++;
    unless ($footprint->[0] =~ m/^\s*$/) {
      #print "$count <start>$footprint->[0]< <len>$footprint->[1]<\n";
      $sum+=$footprint->[1];
    }
  }
  print "sum($seq)=$sum\n";
}

sub multiplarity_from_tex {
  my $footlistref =shift;
  my @seqs=keys(%$footlistref);
  my %multi_s; #sequence count
  my %multi_l; #length sum
  for (my $f=1; $f<scalar(@{$footlistref->{$seqs[0]}}); $f++) {    
    my $multi_s; #collects number of seq contributing to a footprint
    my @multi_l; #collects lengthes contributing to a footprint
    foreach my $seq (keys(%$footlistref)) {
      unless ($footlistref->{$seq}->[$f]->[0] =~ m/^\s*$/) {
	$multi_s++;
	push @multi_l, $footlistref->{$seq}->[$f]->[1];
      }
    }
    $multi_s{$multi_s}++;
    my @sort_l = sort {$a <=> $b} @multi_l;
    $multi_l{$multi_s}+=$sort_l[0];
  }
  return \%multi_s, \%multi_l;
}

sub L_Dist_from_tex {
  my $footlistref =shift;
  my @seqs=keys(%$footlistref);
  my %L_Dist;  #footprint length distribution 
  for (my $f=1; $f<scalar(@{$footlistref->{$seqs[0]}}); $f++) {    
    foreach my $seq (keys(%$footlistref)) {
      unless ($footlistref->{$seq}->[$f]->[0] =~ m/^\s*$/) {
	$L_Dist{$footlistref->{$seq}->[$f]->[1]}++;
      }
    }
  }
  return \%L_Dist;
}

sub print_L_Dist {
  my $L_Distref = shift;
  print "#footprint length distribution\n";
  my @m= keys(%$L_Distref);
  my @index = sort {$b <=> $a} @m;
  for (my $j=1; $j<=$index[0]; $j++) {
    printf "%6d %6d\n", $j, $L_Distref->{$j};
  }
  print "&\n";
}

sub print_multiplarity {
  my @multirefs=@_;
  print "multiplarity\n";
  my @m= keys(%{$multirefs[0]});
  my @index = sort {$b <=> $a} @m;
  my @sum;
  for (my $j=1; $j<=$index[0]; $j++) {
    printf "%5d ", $j;
    my $co=-1;
    foreach my $multi (@multirefs) {
      $co++;
      printf "%7d ", $multi->{$j};
      $sum[$co]+=$multi->{$j};
    }
    print "\n";
  }
  print  "---------------------\n";
  printf "      %7d %7d\n", @sum; 
}

#  takes: to two hash references holding the footprint information
#         (as returned by read_tex) -> just consistent footprints!
#returns: a ref to a hash: key = sequence; value = list of comparison data
#         [0]=length of all footprints in tex1
#         [1]=length of all footprints in tex2
#         [2]=length of all footprints in tex1 that is not covered by tex2
#             as fraction of [0]
#         [3]=length of all footprints in tex2 that is not covered by tex1
#             as fraction of [1]
sub compare_two_tex {
  my ($footref1, $fs1, $footref2, $fs2, $seqsref)=@_;
  my %results;
  foreach my $seq (@$seqsref) {
    my $schnitt; #intersection of ref1 and ref2
    my $all1; #sum of all footprints in ref1
    my $all2; #sum of all footprints in ref2
    my $f1=1;
    my $f2=1;
    my $jott=0;
    my $fover=0;
    my @clusl1; #list of clusters of tex1 overlapping with clusl2
    my @clusl2; #list of clusters of tex2 overlapping with clusl1
    my $concom=0;
    one: for (my $i=$f1; $i<scalar(@{$footref1->{$seq}}); $i++) {
      if ($footref1->{$seq}->[$i]->[0] =~ m/^\s*$/) { next one }
      my $beg1=$footref1->{$seq}->[$i]->[0];
      my $len1=$footref1->{$seq}->[$i]->[1];
      my $end1=$beg1+$len1-1;
#     print "> $i\n";
      $all1+=$len1;
      two: for (my $j=$f2; $j<scalar(@{$footref2->{$seq}}); $j++) { 
	if ($footref2->{$seq}->[$j]->[0] =~ m/^\s*$/) { next }
	my $beg2=$footref2->{$seq}->[$j]->[0];
	my $len2=$footref2->{$seq}->[$j]->[1];
	my $end2=$beg2+$len2-1;
#	print "> > $j\n";
	unless ($j == $jott) {
	  $all2+=$len2;
	}
	$jott=$j;
	my $rellocref=interval::relloc($beg1, $end1, $beg2, $end2);
	if (interval::overlap($beg1, $end1, $beg2, $end2)) {
	  $fover=1;
	  push @clusl1, $fs1->[$i];
	  push @clusl2, $fs2->[$j];
#	  print "$beg2 ($len2) overlap\n";
	  if ($rellocref->[0] == 1) {
	    if ($rellocref->[1] == 1) {
	      $schnitt+=$len1;
	      $f2=$j;
	      next one;
	    }
	    else {
	      $schnitt+=$end2-$beg1+1;
	      next ;
	    }
	  }
	  else {
	    if ($rellocref->[1] == 1) {
	      $schnitt+=$end1-$beg2+1;
	      $f2=$j;
	      next one;
	    }
	    else {
	      $schnitt+=$len2;
	      next;
	    }
	  }
	}
	else {
	  if ($fover) {
	    $concom++;
	    my $pure1=array_intersection(\@clusl1);
	    my $pure2=array_intersection(\@clusl2);
	    print "concom $concom:\n";
	    print " tex1: @$pure1\n";
	    print " tex2: @$pure2\n";
	    @clusl1=();
	    @clusl2=();
	  }
	  if ($rellocref->[1] == 1) { $f2=$j; next one; }
	  else                      {         next      }
	}
      }
    }
#    print "schnitt=$schnitt\n";
    my $only1 = 'n.a.';
    my $only2 = 'n.a.';
    if ($all1) {
      $only1=($all1-$schnitt)/$all1;
    }
    if ($all2) {
      $only2=($all2-$schnitt)/$all2;
    }
    $results{$seq}=[$all1, $all2, $only1, $only2]; 
  }
  return \%results;
}

sub print_compare_two_tex {
  my $resref=shift;
  print "comparison of two footprint lists:\n";
  print "         seq    all1    all2    only1 <->   only2\n";
  foreach my $seq (sort(keys(%$resref))) {
    if ($resref->{$seq}->[2] < $resref->{$seq}->[3]) {
      printf "%-12s %7d %7d  %1.5f --> %1.5f\n", $seq, @{$resref->{$seq}};
    }
    elsif ($resref->{$seq}->[2] > $resref->{$seq}->[3]) {
      printf "%-12s %7d %7d  %1.5f <-- %1.5f\n", $seq, @{$resref->{$seq}};
    }
    else {
      printf "%-12s %7d %7d  %1.5f  =  %1.5f\n", $seq, @{$resref->{$seq}};
    }
  }
}

sub array_intersection {
  my @arrays = @_;
  my %count;
  foreach my $array (@arrays) {
    my %c;
    foreach my $elem (@$array) {
      $c{$elem}++;
    }
    # this is to avoid to count an element twice just because it occures
    # multiple times in the same array
    foreach my $elem (keys(%c)) {
      $count{$elem}++;
    }
  }
  my @back;
  foreach my $elem (keys(%count)) {
    if ($count{$elem} == scalar(@arrays)) {
      push @back, $elem;
    }
  }
  return \@back;
}

sub print_tracker_ps {
  my ($footlistref, $footnameref, $ordaref, $ordhref) = @_;
  my $count=-1;
  my $sum;
  unless (defined($ordaref)) {
    die "Nix iss!\n";
  }
#    my @tmp = keys(%$footlistref);
#    $orderref = \@tmp;
#} 
  print "/FOOTPRINTS [\n";
  foreach my $seq (@$ordaref) {
    my $realseq=$ordhref->{$seq};
    print "  [ 0 1 ($seq)\n    [\n";
    foreach my $footprint (@{$footlistref->{$realseq}}) {
      $count++;
      unless ($footprint->[0] =~ m/^\s*$/) {
	printf "      [ %7d %7d (%-12s) ]\n", $footprint->[0], $footprint->[0]+$footprint->[1]-1, $footnameref->[$count];
      }
    }
    $count=-1;
    print "    ]\n  ]\n";
  }
  print "] def\n";
}



1;


